"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initializeSourceControlManager = exports.sourceControlPush = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _path = require("path");

var _inquirer = require("inquirer");

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

var _chalk = _interopRequireDefault(require("chalk"));

var _exec = require("./exec");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint no-console: 0 */
const generateCloses = closes => {
  let closesString = '';

  for (const close of closes) {
    closesString += `#${close}, `;
  }

  return closesString.substring(0, closesString.length - 2);
};

const generateBreaking = breaking => breaking ? 'BREAKING CHANGE' : '';

const generateStandardCommitMessage = // eslint-disable-next-line
(type, scope, message, closes, breaking) => ({
  title: `${type}(${scope}): ${message}`,
  description: closes.length > 0 ? `${generateCloses(closes)}\n${generateBreaking(breaking)}` : `${generateBreaking(breaking)}`
});

const sourceControlPush = // eslint-disable-next-line
async (branch, newBranch, type, scope, message, closes, breaking = false) => {
  try {
    console.log(_chalk.default.blue(`Checking out ${branch}`));
    const result = await (0, _exec.shellExec)(`git checkout ${newBranch ? '-b' : ''} ${branch}`);
    console.log(_chalk.default.green(result.stdout));
    console.log(_chalk.default.red(result.stderr));
  } catch (err) {
    console.log((0, _chalk.default)(err));
  }

  try {
    console.log(_chalk.default.blue(`Adding all updated files to ${branch}`));
    const result = await (0, _exec.shellExec)(`git add -A`);
    console.log(_chalk.default.green(result.stdout));
    console.log(_chalk.default.red(result.stderr));
  } catch (err) {
    console.log((0, _chalk.default)(err));
  }

  try {
    const commitMessage = generateStandardCommitMessage(type, scope, message, closes, breaking);
    console.log( // eslint-disable-next-line
    _chalk.default.blue(`Adding commit message: \n${commitMessage.title}, ${commitMessage.description}`));
    const result = await (0, _exec.shellExec)( // eslint-disable-next-line
    `git commit -m "${commitMessage.title}" -m "${commitMessage.description}"`);
    console.log(_chalk.default.green(result.stdout));
    console.log(_chalk.default.red(result.stderr));
  } catch (err) {
    console.log((0, _chalk.default)(err));
  }

  try {
    console.log(_chalk.default.blue(`Pushing updated files to ${branch}`));
    const result = await (0, _exec.shellExec)(`git push -u origin ${branch}`);
    console.log(_chalk.default.green(result.stdout));
    console.log(_chalk.default.red(result.stderr));
  } catch (err) {
    console.log((0, _chalk.default)(err));
  }
};

exports.sourceControlPush = sourceControlPush;

const initializeSourceControlManager = // eslint-disable-next-line
async (projectName, repoOwner, branch, withGitIgnore = false) => {
  let result = null;
  let gitAccessToken = null;
  let previousFile = null;
  const applifyDir = (0, _path.join)(process.cwd(), '.applify');

  if (!_fs.default.existsSync(applifyDir)) {
    _fs.default.mkdirSync(applifyDir);
  }

  const gitTokenFile = (0, _path.join)(applifyDir, 'githubToken');

  if (_fs.default.existsSync(gitTokenFile)) {
    previousFile = _fs.default.readFileSync(gitTokenFile, 'utf8');
  } // eslint-disable-next-line


  if (previousFile && previousFile.length > 6) {
    gitAccessToken = previousFile;
  } else {
    const answer = await (0, _inquirer.prompt)([{
      type: 'input',
      message: 'Enter your github personal access token:',
      name: 'gitAccessToken'
    }]);
    gitAccessToken = answer.gitAccessToken;
  }

  _fs.default.writeFileSync(gitTokenFile, `${gitAccessToken}`);

  _fs.default.chmodSync(gitTokenFile, '755');

  if (branch === 'master') {
    try {
      const response = await (0, _nodeFetch.default)('https://api.github.com/user/repos', {
        method: 'POST',
        headers: {
          Authorization: `bearer ${gitAccessToken}`
        },
        body: JSON.stringify({
          name: projectName
        })
      });
      await response.json();
      console.log(_chalk.default.green('Successfully create new remote repository'));
    } catch (err) {
      console.log(_chalk.default.red(err));
    }

    try {
      result = await (0, _exec.shellExec)(`git init`);
      console.log(_chalk.default.red(result.stderr));
      console.log(_chalk.default.green(result.stdout));
    } catch (err) {
      console.log(_chalk.default.red(err));
    }

    try {
      result = await (0, _exec.shellExec)(`git remote add origin git@github.com:${repoOwner}/${projectName}.git`);
    } catch (err) {
      console.log(_chalk.default.yellow(err));
    }
  }

  if (withGitIgnore) {
    const outFileName = (0, _path.join)(process.cwd(), '.gitignore');
    const sourceDir = (0, _path.join)((0, _path.dirname)(process.argv[1]), 'templates');

    _fs.default.copyFileSync((0, _path.join)(sourceDir, '.gitignore'), outFileName);

    console.log(_chalk.default.green('Copied .gitignore'));
  }

  await sourceControlPush(branch, branch === 'master', 'chore', 'tooling', 'Initializing the branch', [], true);
};

exports.initializeSourceControlManager = initializeSourceControlManager;